#!/usr/local/env python
#############################################################################
# Name:         check.py
# Purpose:      Configure and Manage IBM V7000 Unified Storage Appliance
# Maintainer:   Torsten Kurbad
# Copyright:    (c) 2013 iwm-kmrc.de KMRC - Knowledge Media Research Center
# Licence:      GPL
#############################################################################
""" Check adapter to V7000 Unified management module

"""

import os.path

from paramiko import SSHException

from zope.interface import implements
from zope.security.proxy import removeSecurityProxy

from interfaces import ICheckManageV7000UAdapter

class CheckManageV7000UAdapter:
    """ A V7000 Unified management connection test adapter. """

    implements(ICheckManageV7000UAdapter)

    def __init__(self, context):
        self.context = context

    def testConnection(self):
        self.report = []
        adapter = removeSecurityProxy(self.context)
        self.report.append(u'... start check connection')

        self.report.append(u'... trying to connect:')
        self.report.append(u'... hostName              = %s' % adapter.hostName)
        self.report.append(u'... adminUser             = %s' % adapter.adminUser)
        self.report.append(u'... privateKey configured = %r' % ((adapter.adminSSHKey is not None) and (adapter.adminSSHKey != u'')))
        self.report.append(u'...')

        try:
            stdout, stderr = adapter.executeSSHCommand(u'lstime')
            if (stderr is not None) and (stderr != u''):
                self.report.append(u"... <strong>Connection Succeeded but test command 'lstime' failed to execute!</strong>")
                self.report.append(u'... Error message:')
                self.report.append(u'<pre>%s</pre>' % stderr)
            elif (stdout is not None) and (stdout != u''):
                self.report.append(u'... <strong>Connection OK!</strong>')
                self.report.append(u"... Result of 'lstime':")
                self.report.append(u'<pre>%s</pre>' % stdout)
            else:
                self.report.append(u'... <strong>connection FAILED!</strong>')
                return self.report

            self.report.append(u'...')
            stdout, stderr = adapter.executeSSHCommand(u'lsexport')
            if (stderr is not None) and (stderr != u''):
                self.report.append(u"... <strong>Connection Succeeded but test command 'lsexport' failed to execute!</strong>")
                self.report.append(u'... Error message:')
                self.report.append(u'<pre>%s</pre>' % stderr)
            elif (stdout is not None) and (stdout != u''):
                self.report.append(u'... <strong>Connection OK!</strong>')
                self.report.append(u"... Result of 'lsexport':")
                self.report.append(u'<pre>%s</pre>' % stdout)

            # Try to locate profile export in 'lsexport' output
            exportFound = False
            stdOutLines = stdout.splitlines()
            for stdOutLine in stdOutLines:
                stdOutLineFields = stdOutLine.split()
                if (len(stdOutLineFields) > 1) and (adapter.profileExport == stdOutLineFields[1]):
                    exportFound = True
                    break

            if exportFound:
                self.report.append(u"... <strong>Nice work! Profile export could be found in 'lsexport' output!</strong>")
                self.report.append(u'... Shared as: \\\\%s\\%s' % (adapter.hostName, stdOutLineFields[0]))
            else:
                self.report.append(u"... <strong>OH NO! Profile export could not be found in 'lsexport' output!</strong>")
                self.report.append(u'... Your profile export setting seems to be wrong!')

        except SSHException as e:
            self.report.append(u'... <strong>SSH setup is wrong!</strong>')
            self.report.append(u'... Exception: %s' % e)

        except Exception as e:
            self.report.append(u'... <strong>Connection test failed!</strong>')
            self.report.append(u'... Exception: %s' % e)

        self.report.append(u'...')
        if os.path.exists(adapter.profileMountpoint) and os.path.isdir(adapter.profileMountpoint):
            self.report.append(u'... <strong>OK! Local profile mountpoint exists and is a directory.</strong>')
        else:
            self.report.append(u'... <strong>Failed! Local profile mountpoint does not exist or is not a directory!</strong>')

        self.report.append(u'...')
        fstabEntryFound = False
        fstab = u'/etc/fstab'
        if os.path.exists(fstab) and os.path.isfile(fstab):
            try:
                with open(fstab, 'r') as fh:
                    for line in fh:
                        fstabFields = line.split()
                        if len(fstabFields) > 1:
                            if (fstabFields[0].endswith(stdOutLineFields[0])
                                and (fstabFields[1] == adapter.profileMountpoint)):
                                    fstabEntryFound = True
                                    self.report.append(u"... <strong>OK! Entry for local profile mountpoint exists in '/etc/fstab'.</strong>")
                                    break
                if not fstabEntryFound:
                    self.report.append(u"... <strong>Warning! No entry for local profile mountpoint exists in '/etc/fstab'.</strong>")
                    self.report.append(u'... This may lead to complications.')
            except OSError as e:
                self.report.append(u"... <strong>Warning! Could not read '/etc/fstab'.</strong>")
                self.report.append(u'... Exception: %s' % e)
            except Exception as e:
                self.report.append(u"... <strong>Error! Something is wrong with '/etc/fstab'.</strong>")
                self.report.append(u'... Exception: %s' % e)
                
        return self.report
