#!/usr/local/env python
#############################################################################
# Name:         utility.py
# Purpose:      Configure and Manage IBM V7000 Unified Storage Appliance
# Maintainer:   Torsten Kurbad
# Copyright:    (c) 2013 iwm-kmrc.de KMRC - Knowledge Media Research Center
# Licence:      GPL
#############################################################################
""" Adapter to V7000 Unified management module

"""

import paramiko

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

from persistent import Persistent

from zope.interface import implements
from zope.component import adapts
from zope.component import getSiteManager
from zope.component import queryUtility

from zope.schema.fieldproperty import FieldProperty

from zope.i18nmessageid import MessageFactory
_ = MessageFactory(u'v7000uadapter')

from zope.app.component.vocabulary import UtilityVocabulary
from zope.app.container.contained import Contained

from interfaces import IManageV7000UAdapter

# Constants
V7000U_ADAPTER_NAME = u'V7000UAdapter'


class ManageV7000UAdapter(Persistent, Contained, paramiko.SSHClient):
    """ Make v7000u module persistent in ZODB. """

    implements(IManageV7000UAdapter)

    hostName = FieldProperty(IManageV7000UAdapter['hostName'])
    sshHostKey = FieldProperty(IManageV7000UAdapter['sshHostKey'])
    adminUser = FieldProperty(IManageV7000UAdapter['adminUser'])
    adminPassword = FieldProperty(IManageV7000UAdapter['adminPassword'])
    adminSSHKey = FieldProperty(IManageV7000UAdapter['adminSSHKey'])
    profileExport = FieldProperty(IManageV7000UAdapter['profileExport'])
    profileMountpoint = FieldProperty(IManageV7000UAdapter['profileMountpoint'])

    _autoRegister = True

    def __init__(self, hostName = None, sshHostKey = None,
        adminUser = None, adminPassword = None, adminSSHKey = None,
        profileExport = None, profileMountpoint = None):
        """ Initialize v7000uadapter with default values. """
        super(ManageV7000UAdapter, self).__init__()

        if hostName is not None:
            self.hostName = hostName
        if sshHostKey is not None:
            self.sshHostKey = sshHostKey
        if adminUser is not None:
            self.adminUser = adminUser
        if adminPassword is not None:
            self.adminPassword = adminPassword
        if adminSSHKey is not None:
            self.adminSSHKey = adminSSHKey
        if profileExport is not None:
            self.profileExport = profileExport
        if profileMountpoint is not None:
            self.profileMountpoint = profileMountpoint

    def getAutoRegister(self):
        """ Return autoRegister status. """
        return self._autoRegister

    def setAutoRegister(self, value):
        """ Register the adapter as utility. """
        if value:
            siteManager = getSiteManager(self)

            if queryUtility(IManageV7000UAdapter, name = V7000U_ADAPTER_NAME) is None:
                siteManager.registerUtility(self,
                                            IManageV7000UAdapter,
                                            V7000U_ADAPTER_NAME)

        self._autoRegister = value

    autoRegister = property(getAutoRegister, setAutoRegister)

    def executeSSHCommand(self, command, stdInData=None, timeout=None):
        """ Execute a command remotely through SSH on the configured
            IBM Storwize V7000 Unified storage appliance.
        """

        if self.adminSSHKey is None and self.adminPassword is None:
            raise paramiko.PasswordRequiredException(
                _(u'Can not open SSH connection to %s. Neither password '
                u'nor private SSH key have been configured for the admin user.'))

        #
        # Load SSH host key
        #
        # Parse host key from ZMI
        sshHostKeyFields = self.sshHostKey.split(u' ')

        # Do some sanity checks and build a valid ssh-host-key line to add
        if len(sshHostKeyFields) < 2:
            raise paramiko.SSHException(_(u'Invalid SSH Host Key format.'))

        if (len(sshHostKeyFields) > 2) and sshHostKeyFields[0].startswith(self.hostName):
            sshHostKeyFields = sshHostKeyFields[1:]
        sshHostKeyFields = sshHostKeyFields[:2]

        sshHostKeyType, sshHostKeyValue = sshHostKeyFields
        if not sshHostKeyType.startswith('ssh-'):
            raise paramiko.SSHException(_(u'Invalid SSH Host Key format.'))
        if not (sshHostKeyValue.startswith('A') and sshHostKeyValue.endswith('=')):
            raise paramiko.SSHException(_(u'Invalid SSH Host Key format.'))

        sshHostKeyLine = u' '.join([self.hostName, sshHostKeyType, sshHostKeyValue])

        # Add the key
        sshHostKeyEntry = paramiko.hostkeys.HostKeyEntry.from_line(sshHostKeyLine)
        if sshHostKeyEntry is None:
            raise paramiko.SSHException(_(u'SSH Host Key could not be added.'))
        self._host_keys._entries.append(sshHostKeyEntry)

        # Evaluate private key and password
        pkey = None
        password = None
        if (self.adminSSHKey is not None) and (self.adminSSHKey != u''):
            pkeyText = self.adminSSHKey
            if pkeyText.startswith('-----BEGIN DSA PRIVATE KEY-----'):
                pkeyClass = paramiko.DSSKey
            elif pkeyText.startswith('-----BEGIN RSA PRIVATE KEY-----'):
                pkeyClass = paramiko.RSAKey
            else:
                raise paramiko.SSHException(_(u'Invalid SSH Private Key format for admin user.'))

            pkeyObj = StringIO(pkeyText)
            pkey = pkeyClass.from_private_key(pkeyObj)

        if (self.adminPassword is not None) and (self.adminPassword != u''):
            password = self.adminPassword


        # Open a connection
        self.connect(self.hostName,
                    username = self.adminUser,
                    password = password,
                    pkey = pkey,
                    timeout = timeout,
                    allow_agent = False,
                    look_for_keys = False)

        # Execute command
        stdin, stdout, stderr = self.exec_command(command, timeout=timeout)

        # Write to stdin if applicable
        if stdInData is not None:
            stdin.write(stdin)

        stdOutData = stdout.read(32768)
        stdErrData = stderr.read(32768)

        # Close the connection
        self.close()

        # Return result of command execution as tuple of strings (stdout, stderr)
        return stdOutData, stdErrData


class ManageV7000UAdapterVocabulary(UtilityVocabulary):
    interface = IManageV7000UAdapter
    nameOnly = True
